searchData={"items":[{"type":"behaviour","title":"Rabbit.Broker","doc":"A RabbitMQ broker process.\n\nThe broker is a `Supervisor` that encapsulates all the smaller pieces available\nin Rabbit into a single entity. It provides the following:\n\n* Durable connection pooling through `Rabbit.Connection`.\n* Automatic broker configuration through `Rabbit.Topology`.\n* Simple message publishing through `Rabbit.Producer`.\n* Simple message consumption through `Rabbit.ConsumerSupervisor`.\n\nIt's recommended that the documentation for each of these components is read\nwhen utilizing the broker process.\n\nBe aware that the connection for each component of the broker is automatically\nconfigured to use the connection process created by the broker. You dont need\nto configure this yourself.","ref":"Rabbit.Broker.html"},{"type":"behaviour","title":"Example - Rabbit.Broker","doc":"defmodule MyBroker do\n      use Rabbit.Broker\n\n      def start_link(opts \\ []) do\n        Rabbit.Broker.start_link(__MODULE__, opts, name: __MODULE__)\n      end\n\n      # Callbacks\n\n      @impl Rabbit.Broker\n      def init(_type, opts) do\n        # Perform any configuration\n        # You can implement the callback for each component of the broker based\n        # on the first arg.\n        {:ok, opts}\n      end\n\n      @impl Rabbit.Broker\n      def handle_message(message) do\n        # Handle consumed messages\n        {:ack, message}\n      end\n\n      @impl Rabbit.Broker\n      def handle_error(message) do\n        # Handle errors that occur within handle_message/1\n        {:nack, message}\n      end\n    end\n\n    # Start the broker\n    MyBroker.start_link(\n      connection: [uri: \"amqp://guest:guest@127.0.0.1:5672\"],\n      topology: [\n        queues: [[name: \"foo\"], [name: \"bar\"]]\n      ],\n      producer: [pool_size: 10],\n      consumers: [[queue: \"foo\"], [queue: \"bar\", prefetch_count: 10]]\n    )","ref":"Rabbit.Broker.html#module-example"},{"type":"callback","title":"Rabbit.Broker.handle_error/1","doc":"A callback executed by each consumer to handle message exceptions.\n\nPlease see `c:Rabbit.Consumer.handle_error/1` for more information.","ref":"Rabbit.Broker.html#c:handle_error/1"},{"type":"callback","title":"Rabbit.Broker.handle_message/1","doc":"A callback executed by each consumer to handle message consumption.\n\nPlease see `c:Rabbit.Consumer.handle_message/1` for more information.","ref":"Rabbit.Broker.html#c:handle_message/1"},{"type":"callback","title":"Rabbit.Broker.init/2","doc":"A callback executed by each component of the broker.\n\nSeven versions of the callback can be created. The callback is differentiated\nbased on the first arg.\n\n* `:connection_pool` - Callback for the connection pool.\n* `:connection` - Callback for each connection in the pool.\n* `:topology` - Callback for the topology.\n* `:producer_pool` - Callback for the producer pool.\n* `:producer` - Callback for each producer in the pool.\n* `:consumer_supervisor` - Callback for the consumer supervisor.\n* `:consumer` - Callback for each consumer.\n\n      # Initialize the connection pool\n      def init(:connection_pool, opts) do\n        {:ok, opts}\n      end\n\n      # Initialize a single connection\n      def init(:connection, opts) do\n        {:ok, opts}\n      end\n\n      # Initialize the topology\n      def init(:topology, opts) do\n        {:ok, opts}\n      end\n\n      # And so on....\n\nReturning `{:ok, opts}` - where `opts` is a keyword list will cause `start_link/3`\nto return `{:ok, pid}` and the broker to enter its loop.\n\nReturning `:ignore` will cause `start_link/3` to return `:ignore` and the process\nwill exit normally without entering the loop.","ref":"Rabbit.Broker.html#c:init/2"},{"type":"function","title":"Rabbit.Broker.publish/6","doc":"Publishes a message using the provided broker.\n\nThe broker MUST be a broker module, and not a broker PID.\n\nPlease see the `Rabbit.Producer.publish/6` documentation for further details.","ref":"Rabbit.Broker.html#publish/6"},{"type":"function","title":"Rabbit.Broker.start_link/3","doc":"Starts a broker process.","ref":"Rabbit.Broker.html#start_link/3"},{"type":"function","title":"Options - Rabbit.Broker.start_link/3","doc":"* `:connection` - A keyword list of `t:Rabbit.Connection.option/0`.\n  * `:topology` - A keyword list of `t:Rabbit.Topology.option/0`.\n  * `:producer` - A keyword list of `t:Rabbit.Producer.option/0`.\n  * `:consumers` - A list of `t:Rabbit.ConsumerSupervisor.consumers/0`.","ref":"Rabbit.Broker.html#start_link/3-options"},{"type":"function","title":"Server Options - Rabbit.Broker.start_link/3","doc":"You can also provide server options - which are simply the same ones available\nfor `t:GenServer.options/0`.","ref":"Rabbit.Broker.html#start_link/3-server-options"},{"type":"function","title":"Rabbit.Broker.stop/1","doc":"Stops a broker process.","ref":"Rabbit.Broker.html#stop/1"},{"type":"type","title":"Rabbit.Broker.option/0","doc":"","ref":"Rabbit.Broker.html#t:option/0"},{"type":"type","title":"Rabbit.Broker.options/0","doc":"","ref":"Rabbit.Broker.html#t:options/0"},{"type":"type","title":"Rabbit.Broker.t/0","doc":"","ref":"Rabbit.Broker.html#t:t/0"},{"type":"behaviour","title":"Rabbit.Connection","doc":"A RabbitMQ connection pool process.\n\nConnections form the basis of any application that is working with RabbitMQ. A\nconnection module is needed by all the other modules included with Rabbit. They\nwrap around the standard `AMQP.Connection` and provide the following\nbenefits:\n\n* Durability during connection failures through use of exponential backoff.\n* Increased throughput via connection pooling.\n* Subscriptions that assist connection status monitoring.\n* Easy runtime setup through an `c:init/2` callback.","ref":"Rabbit.Connection.html"},{"type":"behaviour","title":"Example - Rabbit.Connection","doc":"# Connection module\n    defmodule MyConnection do\n      use Rabbit.Connection\n\n      def start_link(opts \\\\ []) do\n        Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__)\n      end\n\n      # Callbacks\n\n      @impl Rabbit.Connection\n      def init(_type, opts) do\n        # Perform any runtime configuration\n        {:ok, opts}\n      end\n    end\n\n    # Start the connection\n    MyConnection.start_link()\n\n    # Subscribe to the connection\n    Rabbit.Connection.subscribe(MyConnection)\n\n    receive do\n      {:connected, connection} -> \"hello\"\n    end\n\n    # Stop the connection\n    Rabbit.Connection.stop(MyConnection)\n\n    receive do\n      {:disconnected, reason} -> \"bye\"\n    end","ref":"Rabbit.Connection.html#module-example"},{"type":"function","title":"Rabbit.Connection.alive?/2","doc":"Checks whether a connection is alive within the pool.","ref":"Rabbit.Connection.html#alive?/2"},{"type":"function","title":"Rabbit.Connection.fetch/2","doc":"Fetches a raw `AMQP.Connection` struct from the pool.","ref":"Rabbit.Connection.html#fetch/2"},{"type":"callback","title":"Rabbit.Connection.init/2","doc":"A callback executed by each component of the connection pool.\n\nTwo versions of the callback must be created. One for the pool, and one\nfor the connections. The first argument differentiates the callback.\n\n      # Initialize the pool\n      def init(:connection_pool, opts) do\n        {:ok, opts}\n      end\n\n      # Initialize a single connection\n      def init(:connection, opts) do\n        {:ok, opts}\n      end\n\n\nReturning `{:ok, opts}` - where `opts` is a keyword list of `t:option/0` will,\ncause `start_link/3` to return `{:ok, pid}` and the process to enter its loop.\n\nReturning `:ignore` will cause `start_link/3` to return `:ignore` and the process\nwill exit normally without entering the loop.","ref":"Rabbit.Connection.html#c:init/2"},{"type":"function","title":"Rabbit.Connection.start_link/3","doc":"Starts a connection pool process.","ref":"Rabbit.Connection.html#start_link/3"},{"type":"function","title":"Options - Rabbit.Connection.start_link/3","doc":"* `:uri` - The connection URI. This takes priority over other connection attributes.\n  * `:pool_size` - The number of processes to create for connections - defaults to `1`.\n    Each process consumes a RabbitMQ connection.\n  * `:max_overflow` - Maximum number of temporary workers created when the pool\n    is empty - defaults to `0`.\n  * `:stratgey` - Determines whether checked in workers should be placed first\n    or last in the line of available workers - defaults to `:fifo`.\n  * `:name` - A name that will be displayed in the management UI.\n  * `:username` - The name of a user registered with the broker - defaults to `\"guest\"`.\n  * `:password` - The password of user - defaults to `\"guest\"`.\n  * `:virtual_host` - The name of a virtual host in the broker - defaults to `\"/\"`.\n  * `:host` - The hostname of the broker - defaults to `\"localhost\"`.\n  * `:port` - The port the broker is listening on - defaults to `5672`.\n  * `:channel_max` - The channel_max handshake parameter - defaults to `0`.\n  * `:frame_max` - The frame_max handshake parameter  - defaults to `0`.\n  * `:heartbeat` - The hearbeat interval in seconds - defaults to `10`.\n  * `:connection_timeout` - The connection timeout in milliseconds - defaults to `50000`.\n  * `:retry_backoff` - The amount of time in milliseconds to add between connection retry\n    attempts - defaults to `1_000`.\n  * `:retry_max_delay` - The max amount of time in milliseconds to be used between\n    connection attempts - defaults to `5_000`.\n  * `:ssl_options` - Enable SSL by setting the location to cert files - defaults to `:none`.\n  * `:client_properties` - A list of extra client properties to be sent to the server - defaults to `[]`.\n  * `:socket_options` - Extra socket options. These are appended to the default options.     See http://www.erlang.org/doc/man/inet.html#setopts-2 and http://www.erlang.org/doc/man/gen_tcp.html#connect-4     for descriptions of the available options.","ref":"Rabbit.Connection.html#start_link/3-options"},{"type":"function","title":"Server Options - Rabbit.Connection.start_link/3","doc":"You can also provide server options - which are simply the same ones available\nfor `t:GenServer.options/0`.","ref":"Rabbit.Connection.html#start_link/3-server-options"},{"type":"function","title":"Rabbit.Connection.stop/1","doc":"Stops the connection pool.","ref":"Rabbit.Connection.html#stop/1"},{"type":"function","title":"Rabbit.Connection.subscribe/3","doc":"Subscribes a process to a connection in the pool.\n\nA subscribed process can receive the following messages:\n\n`{:connected, connection}` - where connection is an `AMQP.Connection` struct.\n\nDuring the subscription process, if the connection is alive, this message will\nimmediately be sent. If the connection goes down, and manages to reconnect, this\nmessage will be sent.\n\n`{:disconnected, reason}` - where reason can be any value.\n\nIf the connection goes down, all subscribing processes are sent this message.\nThe connection process will then go through an exponential backoff period until\nconnection is achieved again.","ref":"Rabbit.Connection.html#subscribe/3"},{"type":"function","title":"Rabbit.Connection.transaction/2","doc":"Runs the given function inside a transaction.\n\nThe function must accept a connection pid.","ref":"Rabbit.Connection.html#transaction/2"},{"type":"type","title":"Rabbit.Connection.option/0","doc":"","ref":"Rabbit.Connection.html#t:option/0"},{"type":"type","title":"Rabbit.Connection.options/0","doc":"","ref":"Rabbit.Connection.html#t:options/0"},{"type":"type","title":"Rabbit.Connection.t/0","doc":"","ref":"Rabbit.Connection.html#t:t/0"},{"type":"behaviour","title":"Rabbit.Consumer","doc":"A RabbitMQ consumer process.\n\nConsumers are the \"workers\" of your application. They wrap around the standard\n`AMQP.Channel` and provide the following benefits:\n\n* Durability during connection and channel failures through use of exponential backoff.\n* Easy runtime setup through the `c:init/2` and `c:handle_setup/1` callbacks.\n* Automatic acknowledgements based on the return value of the `c:handle_message/1` callback.\n* Ability to handle exceptions through the `c:handle_error/1` callback.\n* Each message is executed within its own supervised task.\n* Automatic payload decoding based on available serializers and message\n  content type.","ref":"Rabbit.Consumer.html"},{"type":"behaviour","title":"Example - Rabbit.Consumer","doc":"# This is a connection\n    defmodule MyConnection do\n      use Rabbit.Connection\n\n      def start_link(opts \\\\ []) do\n        Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__)\n      end\n\n      # Callbacks\n\n      @impl Rabbit.Connection\n      def init(:connection, opts) do\n        # Perform any runtime configuration\n        {:ok, opts}\n      end\n    end\n\n    # This is a consumer\n    defmodule MyConsumer do\n      use Rabbit.Consumer\n\n      def start_link(opts \\\\ []) do\n        Rabbit.Consumer.start_link(__MODULE__, opts, name: __MODULE__)\n      end\n\n      # Callbacks\n\n      @impl Rabbit.Consumer\n      def init(_type, opts) do\n        # Perform any runtime configuration\n        {:ok, opts}\n      end\n\n      @impl Rabbit.Consumer\n      def handle_setup(state) do\n        # Optional callback to perform exchange or queue setup\n        AMQP.Queue.declare(state.channel, state.queue)\n\n        :ok\n      end\n\n      @impl Rabbit.Consumer\n      def handle_message(message) do\n        # Handle consumed messages\n        {:ack, message}\n      end\n\n      @impl Rabbit.Consumer\n      def handle_error(message) do\n        # Handle errors that occur within handle_message/1\n        {:nack, message}\n      end\n    end\n\n    # Start the connection\n    MyConnection.start_link()\n\n    # Start the consumer\n    MyConsumer.start_link(connection: MyConnection, queue: \"my_queue\", prefetch_count: 20)","ref":"Rabbit.Consumer.html#module-example"},{"type":"behaviour","title":"Serializers - Rabbit.Consumer","doc":"When a message is consumed, its content type is compared to the list of available\nserializers. If a serializer matches the content type, the message will be\nautomatically decoded.\n\nYou can find out more about serializers at `Rabbit.Serializer`.","ref":"Rabbit.Consumer.html#module-serializers"},{"type":"callback","title":"Rabbit.Consumer.handle_error/1","doc":"A callback executed to handle message exceptions.\n\nIf the original `c:handle_message/1` callback raises an error, this callback\nwill be called with the message - but with the `:error_reason` and `:error_stack`\nfields filled.\n\nYou may choose to return the same values as `c:handle_message/1`.","ref":"Rabbit.Consumer.html#c:handle_error/1"},{"type":"callback","title":"Rabbit.Consumer.handle_message/1","doc":"A callback executed to handle message consumption.\n\nThe callback is provided a `Rabbit.Message` struct. You may find more information\nabout the message structure within its own documentation. The message may be\nautomatically decoded based on the content type and available serializers.\n\nYou may choose to ack, nack, or reject the message based on the return value.\n\n* `{:ack, message}` - will acknowledge the message.\n* `{:ack, message, options}` - will acknowledge the message with options.\n* `{:nack, message}` - will negative acknowledge the message.\n* `{:nack, message, options}` - will negative acknowledge the message with options.\n* `{:reject, message}` - will reject the message.\n* `{:reject, message, options}` - will reject the message with options.\n\nIf you don't return one of these values - nothing will be done. This means you\nwill need to manually ack, nack or reject the message if required. Please\nsee the `Rabbit.Message` module for more information.","ref":"Rabbit.Consumer.html#c:handle_message/1"},{"type":"callback","title":"Rabbit.Consumer.handle_setup/1","doc":"An optional callback executed after the channel is open, but before consumption.\n\nThe callback is called with the current state, containing the open channel and queue name if\ngiven. At the most basic, you may want to declare the queue to ensure it's available. This will\nbe entirely application dependent though.\n\n    def handle_setup(state) do\n      AMQP.Queue.declare(state.channel, state.queue)\n\n      :ok\n    end\n\nImportant keys from the state include:\n\n* `:connection` - the `Rabbit.Connection` module in use.\n* `:channel` - the `AMQP.Channel` open for this consumer.\n* `:queue` - the queue name.\n* `:setup_opts` - as provided to `start_link/3`.\n\nReturn either `:ok` or `{:ok, new_state}` for success, the latter will update the state.\n\nIf another value is returned it will be marked as failed, and the consumer will attempt to go\nthrough the connection setup process again.\n\nAlternatively, you could use a `Rabbit.Topology` process to perform this\nsetup work. Please see its docs for more information.","ref":"Rabbit.Consumer.html#c:handle_setup/1"},{"type":"callback","title":"Rabbit.Consumer.init/2","doc":"A callback executed when the consumer is started.\n\nReturning `{:ok, opts}` - where `opts` is a keyword list of `t:option()` will,\ncause `start_link/3` to return `{:ok, pid}` and the process to enter its loop.\n\nReturning `:ignore` will cause `start_link/3` to return `:ignore` and the process\nwill exit normally without entering the loop.","ref":"Rabbit.Consumer.html#c:init/2"},{"type":"function","title":"Rabbit.Consumer.start_link/3","doc":"Starts a consumer process.","ref":"Rabbit.Consumer.html#start_link/3"},{"type":"function","title":"Options - Rabbit.Consumer.start_link/3","doc":"* `:connection` - A `Rabbit.Connection` process.\n  * `:queue` - The queue to consume messages from.\n  * `:prefetch_count` - The basic unit of concurrency for a given consumer - defaults to `0`.\n  * `:prefetch_size` - The prefetch window size in octets - defaults to `0`, meaning\n    no specific limit.\n  * `:consumer_tag` - The identifier of the consumer. If empty, one will be\n    generated by the server.\n  * `:no_local` - A boolean representing whether messages should not be sent to the\n    same connection that published them - defaults to `false`.\n  * `:no_ack` - A boolean representing whether acknowledgements are not required for\n    messages - defaults to `false`.\n  * `:exclusive` - A boolean representing whether only this consumer can access\n    the queue - defaults to `false`.\n  * `:nowait` - A boolean representing whether the server should not respond to\n    methods - defaults to `false`.\n  * `:arguments` - A set of arguments for the consumer.\n  * `:custom_meta` - A map of custom data that will be included in each `Rabbit.Message`\n    handled by the consumer.\n  * `:setup_opts` - A keyword list of custom options for use in `c:handle_setup/1`.\n  * `:workers` - The number of workers available to process messages - defaults\n    to `System.schedulers_online()`.\n  * `:timeout` - The timeout in milliseconds for each message processed - defaults\n    to `60_000`.","ref":"Rabbit.Consumer.html#start_link/3-options"},{"type":"function","title":"Server Options - Rabbit.Consumer.start_link/3","doc":"You can also provide server options - which are simply the same ones available\nfor `t:GenServer.options/0`.","ref":"Rabbit.Consumer.html#start_link/3-server-options"},{"type":"function","title":"Rabbit.Consumer.stop/1","doc":"Stops a consumer process.","ref":"Rabbit.Consumer.html#stop/1"},{"type":"type","title":"Rabbit.Consumer.action_options/0","doc":"","ref":"Rabbit.Consumer.html#t:action_options/0"},{"type":"type","title":"Rabbit.Consumer.delivery_tag/0","doc":"","ref":"Rabbit.Consumer.html#t:delivery_tag/0"},{"type":"type","title":"Rabbit.Consumer.message_response/0","doc":"","ref":"Rabbit.Consumer.html#t:message_response/0"},{"type":"type","title":"Rabbit.Consumer.option/0","doc":"","ref":"Rabbit.Consumer.html#t:option/0"},{"type":"type","title":"Rabbit.Consumer.options/0","doc":"","ref":"Rabbit.Consumer.html#t:options/0"},{"type":"type","title":"Rabbit.Consumer.setup_options/0","doc":"","ref":"Rabbit.Consumer.html#t:setup_options/0"},{"type":"type","title":"Rabbit.Consumer.t/0","doc":"","ref":"Rabbit.Consumer.html#t:t/0"},{"type":"behaviour","title":"Rabbit.ConsumerSupervisor","doc":"A RabbitMQ consumer supervisor process.\n\nThis allows starting and supervising multiple `Rabbit.Consumer` processes with\nease. Rather than creating a module for each consumer and implementing repetitive\nlogic - the same callbacks are used across all child consumers.","ref":"Rabbit.ConsumerSupervisor.html"},{"type":"behaviour","title":"Example - Rabbit.ConsumerSupervisor","doc":"# This is a connection\n    defmodule MyConnection do\n      use Rabbit.Connection\n\n      def start_link(opts \\\\ []) do\n        Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__)\n      end\n\n      # Callbacks\n\n      @impl Rabbit.Connection\n      def init(:connection, opts) do\n        # Perform any runtime configuration\n        {:ok, opts}\n      end\n    end\n\n    # This is a consumer supervisor\n    defmodule MyConsumers do\n      use Rabbit.ConsumerSupervisor\n\n      def start_link(consumers \\\\ []) do\n        Rabbit.ConsumerSupervisor.start_link(__MODULE__, consumers, name: __MODULE__)\n      end\n\n      # Callbacks\n\n      @impl Rabbit.ConsumerSupervisor\n      def init(:consumer_supervisor, _consumers) do\n        # Perform any runtime configuration for the supervisor\n        consumers = [\n          [connection: MyConnection, queue: \"my_queue1\", prefetch_count: 5],\n          [connection: MyConnection, queue: \"my_queue2\", prefetch_count: 10]\n        ]\n\n        {:ok, consumers}\n      end\n\n      def init(:consumer, opts) do\n        # Perform any runtime configuration per consumer\n        {:ok, opts}\n      end\n\n      @impl Rabbit.ConsumerSupervisor\n      def handle_setup(channel, queue) do\n        # Perform exchange or queue setup per consumer\n        AMQP.Queue.declare(channel, queue)\n\n        :ok\n      end\n\n      @impl Rabbit.ConsumerSupervisor\n      def handle_message(message) do\n        # Handle messages per consumer\n        {:ack, message}\n      end\n\n      @impl Rabbit.ConsumerSupervisor\n      def handle_error(message) do\n        # Handle errors that occur per consumer\n        {:nack, message}\n      end\n    end\n\n    # Start the connection\n    MyConnection.start_link()\n\n    # Start the consumers\n    MyConsumers.start_link()","ref":"Rabbit.ConsumerSupervisor.html#module-example"},{"type":"callback","title":"Rabbit.ConsumerSupervisor.handle_error/1","doc":"A callback executed by each consumer to handle message exceptions.\n\nPlease see `c:Rabbit.Consumer.handle_error/1` for more information.","ref":"Rabbit.ConsumerSupervisor.html#c:handle_error/1"},{"type":"callback","title":"Rabbit.ConsumerSupervisor.handle_message/1","doc":"A callback executed by each consumer to handle message consumption.\n\nPlease see `c:Rabbit.Consumer.handle_message/1` for more information.","ref":"Rabbit.ConsumerSupervisor.html#c:handle_message/1"},{"type":"callback","title":"Rabbit.ConsumerSupervisor.handle_setup/1","doc":"A callback executed by each consumer after the channel is open, but before\nconsumption.\n\nPlease see `c:Rabbit.Consumer.handle_setup/1` for more information.","ref":"Rabbit.ConsumerSupervisor.html#c:handle_setup/1"},{"type":"callback","title":"Rabbit.ConsumerSupervisor.init/2","doc":"A callback executed by each component of the consumer supervisor.\n\nTwo versions of the callback must be created. One for the supervisor, and one\nfor the consumers. The first argument differentiates the callback.\n\n      # Initialize the supervisor\n      def init(:consumer_supervisor, consumers) do\n        {:ok, consumers}\n      end\n\n      # Initialize a single consumer\n      def init(:consumer, opts) do\n        {:ok, opts}\n      end\n\nReturning `{:ok, consumers}` - where `consumers` is a list of `t:Rabbit.Consumer.options/0`\nwill, cause `start_link/3` to return `{:ok, pid}` and the supervisor to enter its loop.\n\nReturning `{:ok, opts}` - where `opts` is a keyword list of `t:Rabbit.Consumer.option/0` will,\ncause `start_link/3` to return `{:ok, pid}` and the consumer to enter its loop.\n\nReturning `:ignore` will cause `start_link/3` to return `:ignore` and the process\nwill exit normally without entering the loop.","ref":"Rabbit.ConsumerSupervisor.html#c:init/2"},{"type":"function","title":"Rabbit.ConsumerSupervisor.start_link/3","doc":"Starts a consumer supervisor process.\n\nThe list of consumers should represent a list of `t:Rabbit.Consumer.options/0`.","ref":"Rabbit.ConsumerSupervisor.html#start_link/3"},{"type":"function","title":"Server Options - Rabbit.ConsumerSupervisor.start_link/3","doc":"You can also provide server options - which are simply the same ones available\nfor `t:GenServer.options/0`.","ref":"Rabbit.ConsumerSupervisor.html#start_link/3-server-options"},{"type":"type","title":"Rabbit.ConsumerSupervisor.consumers/0","doc":"","ref":"Rabbit.ConsumerSupervisor.html#t:consumers/0"},{"type":"type","title":"Rabbit.ConsumerSupervisor.t/0","doc":"","ref":"Rabbit.ConsumerSupervisor.html#t:t/0"},{"type":"module","title":"Rabbit.Message","doc":"A message consumed by a `Rabbit.Consumer`.\n\nAfter starting a consumer, any message passed to the `c:Rabbit.Consumer.handle_message/1`\ncallback will be wrapped in a messsage struct. The struct has the following\nfields:\n\n* `:consumer` - The PID of the consumer process.\n* `:module` - The module of the consumer process.\n* `:channel` - The `AMQP.Channel` being used by the consumer.\n* `:payload` - The raw payload of the message.\n* `:decoded_payload` - If the message has a content type - this will be the\n  payload decoded using the applicable serializer.\n* `:meta` - The metadata sent when publishing or set by the broker.\n* `:custom_meta` - The custom metadata included when starting a consumer.\n* `:error_reason` - The reason for any error that occurs during the message\n  handling callback.\n* `:error_stack` - The stacktrace that might accompany the error.","ref":"Rabbit.Message.html"},{"type":"function","title":"Rabbit.Message.ack/2","doc":"Awknowledges a message.","ref":"Rabbit.Message.html#ack/2"},{"type":"function","title":"Options - Rabbit.Message.ack/2","doc":"* `:multiple` -  If  `true`, all messages up to the one specified by its\n  `delivery_tag` are considered acknowledged by the server.","ref":"Rabbit.Message.html#ack/2-options"},{"type":"function","title":"Rabbit.Message.nack/2","doc":"Negative awknowledges a message.","ref":"Rabbit.Message.html#nack/2"},{"type":"function","title":"Options - Rabbit.Message.nack/2","doc":"* `:multiple` -  If `true`, all messages up to the one specified by it\n  `delivery_tag` are considered acknowledged by the server.\n  * `:requeue` - If `true`, the message will be returned to the queue and redelivered\n    to the next available consumer.","ref":"Rabbit.Message.html#nack/2-options"},{"type":"function","title":"Rabbit.Message.new/6","doc":"Creates a new message struct.","ref":"Rabbit.Message.html#new/6"},{"type":"function","title":"Rabbit.Message.reject/2","doc":"Rejects a message.","ref":"Rabbit.Message.html#reject/2"},{"type":"function","title":"Options - Rabbit.Message.reject/2","doc":"* `:requeue` - If `true`, the message will be returned to the queue and redelivered\n    to the next available consumer.","ref":"Rabbit.Message.html#reject/2-options"},{"type":"type","title":"Rabbit.Message.t/0","doc":"","ref":"Rabbit.Message.html#t:t/0"},{"type":"behaviour","title":"Rabbit.Producer","doc":"A RabbitMQ producer process.\n\nProducers are needed to publish any messages to RabbitMQ. They wrap around the\nstandard `AMQP.Channel` and provide the following benefits:\n\n* Durability during connection and channel failures through use of exponential backoff.\n* Channel pooling for increased publishing performance.\n* Easy runtime setup through an `c:init/2` and `c:handle_setup/1` callbacks.\n* Simplification of standard publishing options.\n* Automatic payload encoding based on available serializers and message\n  content type.","ref":"Rabbit.Producer.html"},{"type":"behaviour","title":"Example - Rabbit.Producer","doc":"# This is a connection\n    defmodule MyConnection do\n      use Rabbit.Connection\n\n      def start_link(opts \\\\ []) do\n        Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__)\n      end\n\n      # Callbacks\n\n      @impl Rabbit.Connection\n      def init(_type, opts) do\n        {:ok, opts}\n      end\n    end\n\n    # This is a producer\n    defmodule MyProducer do\n      use Rabbit.Producer\n\n      def start_link(opts \\\\ []) do\n        Rabbit.Producer.start_link(__MODULE__, opts, name: __MODULE__)\n      end\n\n      # Callbacks\n\n      @impl Rabbit.Producer\n      def init(:producer_pool, opts) do\n        # Perform any runtime configuration for the pool\n        {:ok, opts}\n      end\n\n      def init(:producer, opts) do\n        # Perform any runtime configuration per producer\n        {:ok, opts}\n      end\n    end\n\n    # Start the connection\n    MyConnection.start_link()\n\n    # Start the producer\n    MyProducer.start_link(connection: MyConnection, publish_opts: [content_type: \"application/json\"])\n\n    # Publish a message\n    Rabbit.Producer.publish(MyProducer, \"\", \"my_queue\", %{foo: \"bar\"})","ref":"Rabbit.Producer.html#module-example"},{"type":"behaviour","title":"Serializers - Rabbit.Producer","doc":"When a message is published, its content type is compared to the list of available\nserializers. If a serializer matches the content type, the message will be\nautomatically encoded.\n\nYou can find out more about serializers at `Rabbit.Serializer`.","ref":"Rabbit.Producer.html#module-serializers"},{"type":"callback","title":"Rabbit.Producer.handle_setup/1","doc":"An optional callback executed after the channel is open for each producer.\n\nThe callback is called with the current state. At the most basic, you may want to declare queues\nthat you will be publishing to.\n\n    def handle_setup(state) do\n      AMQP.Queue.declare(state.channel, \"some_queue\")\n\n      :ok\n    end\n\nImportant keys in the state include:\n\n* `:connection` - the `Rabbit.Connection` process.\n* `:channel` - the opened `AMQP.Channel` channel.\n* `:setup_opts` - options provided undert the key `:setup_opts` to `start_link/3`.\n\nThe callback must return an `:ok` atom or `{:ok, state}` tuple - otherise it will be marked as\nfailed, and the producer will attempt to go through the channel setup process again.\n\nAlternatively, you could use a `Rabbit.Topology` process to perform this\nsetup work. Please see its docs for more information.","ref":"Rabbit.Producer.html#c:handle_setup/1"},{"type":"callback","title":"Rabbit.Producer.init/2","doc":"A callback executed by each component of the producer pool.\n\nTwo versions of the callback must be created. One for the pool, and one\nfor the producers. The first argument differentiates the callback.\n\n      # Initialize the pool\n      def init(:producer_pool, opts) do\n        {:ok, opts}\n      end\n\n      # Initialize a single producer\n      def init(:producer, opts) do\n        {:ok, opts}\n      end\n\nReturning `{:ok, opts}` - where `opts` is a keyword list of `t:option()` will,\ncause `start_link/3` to return `{:ok, pid}` and the process to enter its loop.\n\nReturning `:ignore` will cause `start_link/3` to return `:ignore` and the process\nwill exit normally without entering the loop","ref":"Rabbit.Producer.html#c:init/2"},{"type":"function","title":"Rabbit.Producer.publish/6","doc":"Publishes a message using the provided producer.\n\nAll publishing options can be provided to the producer during process start.\nThey would then be used as defaults during publishing. Options provided to this\nfunction would overwrite any defaults the producer has.","ref":"Rabbit.Producer.html#publish/6"},{"type":"function","title":"Serializers - Rabbit.Producer.publish/6","doc":"If a `content_type` is provided as an option - and it matches one of the available\nserializers, the payload will be automatically encoded using that serializer.\n\nFor example, we could automatically encode our payload to JSON if we do the\nfollowing:\n\n      Rabbit.Producer.publish(MyProducer, \"\", \"my_queue\", %{foo: \"bar\"}, content_type: \"application/json\")\n\nPlease see the documention at `Rabbit.Serializer` for more information.","ref":"Rabbit.Producer.html#publish/6-serializers"},{"type":"function","title":"Options - Rabbit.Producer.publish/6","doc":"* `:mandatory` - If set, returns an error if the broker can't route the message\n    to a queue - defaults to `false`.\n  * `:immediate` - If set, returns an error if the broker can't deliver the message\n    to a consumer immediately - defaults to `false`.\n  * `:content_type` - MIME Content type.\n  * `:content_encoding` - MIME Content encoding.\n  * `:headers` - Message headers. Can be used with headers Exchanges.\n  * `:persistent` - If set, uses persistent delivery mode. Messages marked as\n    persistent that are delivered to durable queues will be logged to disk.\n  * `:correlation_id` - Application correlation identifier.\n  * `:priority` - Message priority, ranging from 0 to 9.\n  * `:reply_to` - Name of the reply queue.\n  * `:expiration` - How long the message is valid (in milliseconds).\n  * `:message_id` - Message identifier.\n  * `:timestamp` - Timestamp associated with this message (epoch time).\n  * `:type` - Message type as a string.\n  * `:user_id` - Creating user ID. RabbitMQ will validate this against the active\n    connection user.\n  * `:app_id` - Publishing application ID.","ref":"Rabbit.Producer.html#publish/6-options"},{"type":"function","title":"Rabbit.Producer.start_link/3","doc":"Starts a producer process.","ref":"Rabbit.Producer.html#start_link/3"},{"type":"function","title":"Options - Rabbit.Producer.start_link/3","doc":"* `:connection` - A `Rabbit.Connection` process.\n  * `:pool_size` - The number of processes to create for producers - defaults to `1`.\n    Each process consumes a RabbitMQ channel.\n  * `:max_overflow` - Maximum number of temporary workers created when the pool\n    is empty - defaults to `0`.\n  * `:strategy` - Determines whether checked in workers should be placed first\n    or last in the line of available workers - defaults to `:lifo`.\n  * `:sync_start` - Boolean representing whether to establish the connection\n    and channel synchronously - defaults to `true`.\n  * `:sync_start_delay` - The amount of time in milliseconds to sleep between\n    sync start attempts - defaults to `50`.\n  * `:sync_start_max` - The max amount of sync start attempts that will occur\n    before proceeding with async start - defaults to `100`.\n  * `:publish_opts` - Any `t:publish_option/0` that is automatically set as a\n    default option value when calling `publish/6`.\n  * `:setup_opts` - a keyword list of values to provide to `c:handle_setup/1`.","ref":"Rabbit.Producer.html#start_link/3-options"},{"type":"function","title":"Server Options - Rabbit.Producer.start_link/3","doc":"You can also provide server options - which are simply the same ones available\nfor `t:GenServer.options/0`.","ref":"Rabbit.Producer.html#start_link/3-server-options"},{"type":"function","title":"Rabbit.Producer.stop/1","doc":"Stops a producer process.","ref":"Rabbit.Producer.html#stop/1"},{"type":"function","title":"Rabbit.Producer.transaction/2","doc":"Runs the given function inside a transaction.\n\nThe function must accept a producer child pid.","ref":"Rabbit.Producer.html#transaction/2"},{"type":"type","title":"Rabbit.Producer.exchange/0","doc":"","ref":"Rabbit.Producer.html#t:exchange/0"},{"type":"type","title":"Rabbit.Producer.message/0","doc":"","ref":"Rabbit.Producer.html#t:message/0"},{"type":"type","title":"Rabbit.Producer.option/0","doc":"","ref":"Rabbit.Producer.html#t:option/0"},{"type":"type","title":"Rabbit.Producer.options/0","doc":"","ref":"Rabbit.Producer.html#t:options/0"},{"type":"type","title":"Rabbit.Producer.publish_option/0","doc":"","ref":"Rabbit.Producer.html#t:publish_option/0"},{"type":"type","title":"Rabbit.Producer.publish_options/0","doc":"","ref":"Rabbit.Producer.html#t:publish_options/0"},{"type":"type","title":"Rabbit.Producer.routing_key/0","doc":"","ref":"Rabbit.Producer.html#t:routing_key/0"},{"type":"type","title":"Rabbit.Producer.setup_options/0","doc":"","ref":"Rabbit.Producer.html#t:setup_options/0"},{"type":"type","title":"Rabbit.Producer.t/0","doc":"","ref":"Rabbit.Producer.html#t:t/0"},{"type":"behaviour","title":"Rabbit.Serializer","doc":"A behaviour to implement serializers.\n\nTo create a serializer, you just need to implement the `c:encode/1` and `c:decode/1`\ncallbacks,","ref":"Rabbit.Serializer.html"},{"type":"behaviour","title":"Example - Rabbit.Serializer","doc":"defmodule MySerializer do\n        @behaviour Rabbit.Serializer\n\n        @impl true\n        def encode(data) do\n          # Create a binary from data\n        end\n\n        @impl true\n        def decode(binary) do\n          # Create data from a binary\n        end\n      end","ref":"Rabbit.Serializer.html#module-example"},{"type":"behaviour","title":"Default Serializers - Rabbit.Serializer","doc":"By default, Rabbit comes with serializers for the following content types:\n\n* `\"application/etf\"` - built in erlang term format.\n* `\"application/json\"` - requires the `Jason` library to be added.\n\nYou can modify the available serializers through application config:\n\n      config :rabbit,\n        serializers: %{\n          \"application/custom-type\" => MySerializer\n        }","ref":"Rabbit.Serializer.html#module-default-serializers"},{"type":"callback","title":"Rabbit.Serializer.decode/1","doc":"Callback invoked to decode the given binary to data.","ref":"Rabbit.Serializer.html#c:decode/1"},{"type":"callback","title":"Rabbit.Serializer.encode/1","doc":"Callback invoked to encode the given data to a binary.","ref":"Rabbit.Serializer.html#c:encode/1"},{"type":"type","title":"Rabbit.Serializer.t/0","doc":"","ref":"Rabbit.Serializer.html#t:t/0"},{"type":"behaviour","title":"Rabbit.Topology","doc":"A RabbitMQ topology process.\n\nThis is a blocking process that can be used to declare exchanges, queues, and\nbindings. Basically - performing any RabbitMQ setup required by your application.\nIt should be added to your supervision tree before any producers or consumers.\n\nBoth `Rabbit.Consumer` and `Rabbit.ConsumerSupervisor` have the `handle_setup/1`\ncallback, which can be used to perform any queue, exchange or binding work as\nwell. But if you have more complex requirements, this module can be used.","ref":"Rabbit.Topology.html"},{"type":"behaviour","title":"Example - Rabbit.Topology","doc":"# This is a connection\n    defmodule MyConnection do\n      use Rabbit.Connection\n\n      def start_link(opts \\ []) do\n        Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__)\n      end\n\n      # Callbacks\n\n      @impl Rabbit.Connection\n      def init(:connection, opts) do\n        # Perform any runtime configuration\n        {:ok, opts}\n      end\n    end\n\n    # This is a topology\n    defmodule MyTopology do\n      use Rabbit.Topology\n\n      def start_link(opts \\ []) do\n        Rabbit.Topology.start_link(__MODULE__, opts, name: __MODULE__)\n      end\n\n      # Callbacks\n\n      @impl Rabbit.Topology\n      def init(_type, opts) do\n        # Perform any runtime configuration\n        {:ok, opts}\n      end\n    end\n\n    # Start the connection\n    MyConnection.start_link()\n\n    # Start the topology\n    MyTopology.start_link(\n      connection: MyConnection,\n      exchanges: [\n        [name: \"my_exchange_1\"],\n        [name: \"my_exchange_2\", type: :fanout],\n      ],\n      queues: [\n        [name: \"my_queue_1\"],\n        [name: \"my_queue_2\", durable: true],\n      ],\n      bindings: [\n        [type: :queue, source: \"my_exchange_1\", destination: \"my_queue_1\"],\n        [type: :exchange, source: \"my_exchange_2\", destination: \"my_exchange_1\"],\n      ]\n    )","ref":"Rabbit.Topology.html#module-example"},{"type":"callback","title":"Rabbit.Topology.init/2","doc":"A callback executed when the topology is started.\n\nReturning `{:ok, opts}` - where `opts` is a keyword list of `t:option/0` will\ncause `start_link/3` to return `{:ok, pid}` and the process to enter its loop.\n\nReturning `:ignore` will cause `start_link/3` to return `:ignore` and the process\nwill exit normally without entering the loop","ref":"Rabbit.Topology.html#c:init/2"},{"type":"function","title":"Rabbit.Topology.start_link/3","doc":"Starts a toplogy process.","ref":"Rabbit.Topology.html#start_link/3"},{"type":"function","title":"Options - Rabbit.Topology.start_link/3","doc":"* `:connection` - A `Rabbit.Connection` process.\n  * `:exchanges` - A list of exchanges to declare. Please see [Exchanges](#start_link/3-exchanges).\n  * `:queues` - A list of queues to declare. Please see [Queues](#start_link/3-queues).\n  * `:bindings` - A list of bindings to declare. Please see [Bindings](#start_link/3-bindings).\n  * `:retry_delay` - The amount of time in milliseconds to delay between attempts\n    to fetch a connection from the connection process - defaults to `100`.\n  * `:retry_max` - The max amount of connection retries that will be attempted before\n    returning an error - defaults to `25`.","ref":"Rabbit.Topology.html#start_link/3-options"},{"type":"function","title":"Exchanges - Rabbit.Topology.start_link/3","doc":"Declaring exchanges is done by providing a list of keyword options. The options\ninclude:\n\n  * `:name` - The name of the exchange.\n  * `:type` - The type of the exchange - one of `:direct`, `:fanout`, `:topic`,\n    `:match` or `:headers` - defaults to `:direct`. Custom types can also be provided.\n  * `:durable` - Whether the exchange is durable across broker restarts - defaults to `false`.\n  * `:auto_delete` - Deletes the exchange once all queues unbind from it - defaults to `false`.\n  * `:passive` - Returns an error if the exchange does not already exist - defaults to `false`.\n  * `:internal` - If set, the exchange may not be used directly by publishers, but only when\n     bound to other exchanges. Internal exchanges are used to construct wiring that is not visible\n     to applications - defaults to `false`.\n\nBelow is an example of exchange options:\n\n    [\n      [name: \"my_exchange_1\"],\n      [name: \"my_exchange_2\", type: :fanout, durable: true],\n    ]","ref":"Rabbit.Topology.html#start_link/3-exchanges"},{"type":"function","title":"Queues - Rabbit.Topology.start_link/3","doc":"Declaring queues is done by providing a list of keyword options. The options\ninclude:\n\n  * `:name` - The name of the queue.\n  * `:durable` - Whether the queue is durable across broker restarts - defaults to `false`.\n  * `:auto_delete` - Deletes the queue once all consumers disconnect - defaults to `false`.\n  * `:passive` - Returns an error if the queue does not already exist - defaults to `false`.\n  * `:exclusive` - If set, only one consumer can consume from the queue - defaults to `false`.\n\nBelow is an example of queue options:\n\n    [\n      [name: \"my_queue_1\"],\n      [name: \"my_queue_2\", durable: true],\n    ]","ref":"Rabbit.Topology.html#start_link/3-queues"},{"type":"function","title":"Bindings - Rabbit.Topology.start_link/3","doc":"Declaring bindings is done by providing a list of keyword options. The options\ninclude:\n\n  * `:type` - The type of the destination - one of `:exchange` or `:queue`.\n  * `:source` - The source of the binding.\n  * `:destination` - The destination of the binding.\n  * `:routing_key` - The routing key of the binding.\n\nBelow is an example of binding options:\n\n    [\n      [type: :queue, source: \"my_exchange_1\", destination: \"my_queue_1\"],\n      [type: :exchange, source: \"my_exchange_2\", destination: \"my_exchange_1\"]\n    ]","ref":"Rabbit.Topology.html#start_link/3-bindings"},{"type":"function","title":"Server Options - Rabbit.Topology.start_link/3","doc":"You can also provide server options - which are simply the same ones available\nfor `t:GenServer.options/0`.","ref":"Rabbit.Topology.html#start_link/3-server-options"},{"type":"type","title":"Rabbit.Topology.binding/0","doc":"","ref":"Rabbit.Topology.html#t:binding/0"},{"type":"type","title":"Rabbit.Topology.exchange/0","doc":"","ref":"Rabbit.Topology.html#t:exchange/0"},{"type":"type","title":"Rabbit.Topology.option/0","doc":"","ref":"Rabbit.Topology.html#t:option/0"},{"type":"type","title":"Rabbit.Topology.options/0","doc":"","ref":"Rabbit.Topology.html#t:options/0"},{"type":"type","title":"Rabbit.Topology.queue/0","doc":"","ref":"Rabbit.Topology.html#t:queue/0"},{"type":"type","title":"Rabbit.Topology.t/0","doc":"","ref":"Rabbit.Topology.html#t:t/0"},{"type":"extras","title":"Rabbit","doc":"# Rabbit\n\n[![Build Status](https://travis-ci.org/nsweeting/rabbit.svg?branch=master)](https://travis-ci.org/nsweeting/rabbit)\n[![Rabbit Version](https://img.shields.io/hexpm/v/rabbit.svg)](https://hex.pm/packages/rabbit)\n\nRabbit is a set of tools for building applications with RabbitMQ.","ref":"readme.html"},{"type":"extras","title":"Installation - Rabbit","doc":"The package can be installed by adding `rabbit` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:rabbit, \"~> 0.20\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Documentation - Rabbit","doc":"Please see [HexDocs](https://hexdocs.pm/rabbit) for additional documentation.\n\n## [Connections](https://hexdocs.pm/rabbit/Rabbit.Connection.html)\n\nConnections form the basis of any application that is working with RabbitMQ. A\nconnection module is needed by all the other modules included with Rabbit.\n\nUpon start, a connection will automatically create a pool of RabbitMQ connections\nto utilize.\n\n```elixir\ndefmodule MyConnection do\n  use Rabbit.Connection\n\n  def start_link(opts \\\\ []) do\n    Rabbit.Connection.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n\n  # Callbacks\n\n  @impl Rabbit.Connection\n  def init(:connection_pool, opts) do\n    # Perform runtime pool config\n    {:ok, opts}\n  end\n\n  def init(:connection, opts) do\n    # Perform runtime connection config\n    uri = System.get_env(\"RABBITMQ_URI\") || \"amqp://guest:guest@127.0.0.1:5672\"\n    opts = Keyword.put(opts, :uri, uri)\n\n    {:ok, opts}\n  end\nend\n\nMyConnection.start_link()\n```\n\n## [Consumers](https://hexdocs.pm/rabbit/Rabbit.Consumer.html)\n\nConsumers are the \"workers\" of your application. They must be provided a connection\nmodule and queue to consume. Every message received is then passed along to your\n`handle_message/1` callback within its own process.\n\nYou can optionally implement the `handle_setup/2` callback to perform any work\nneeded to declare queues/exchanges/bindings.\n\n```elixir\ndefmodule MyConsumer do\n  use Rabbit.Consumer\n\n  def start_link(opts \\\\ []) do\n    Rabbit.Consumer.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n\n  # Callbacks\n\n  @impl Rabbit.Consumer\n  def init(:consumer, opts) do\n    # Perform runtime config\n    {:ok, opts}\n  end\n\n  @impl Rabbit.Consumer\n  def handle_setup(state) do\n    # Optional callback to perform any exchange or queue setup\n    AMQP.Queue.declare(state.channel, state.queue)\n    :ok\n  end\n\n  @impl Rabbit.Consumer\n  def handle_message(message) do\n    # Handle message consumption\n    IO.inspect(message.payload)\n    {:ack, message}\n  end\n\n  @impl Rabbit.Consumer\n  def handle_error(message) do\n    # Handle message errors\n    {:nack, message}\n  end\nend\n\nMyConsumer.start_link(connection: MyConnection, queue: \"my_queue\", prefetch_count: 10)\n```\n\n## [Consumer Supervisors](https://hexdocs.pm/rabbit/Rabbit.ConsumerSupervisor.html)\n\nConsumer supervisors provide an easy way to start and supervise multiple consumer\nprocesses. Rather than creating a module for each consumer and implementing\nrepetitive logic - the same callbacks are used across all consumers.\n\n```elixir\ndefmodule MyConsumerSupervisor do\n  use Rabbit.ConsumerSupervisor\n\n  def start_link(consumers \\\\ []) do\n    Rabbit.ConsumerSupervisor.start_link(__MODULE__, consumers, name: __MODULE__)\n  end\n\n  # Callbacks\n\n  @impl Rabbit.ConsumerSupervisor\n  def init(:consumer_supervisor, _consumers) do\n    # Perform runtime config for the consumer supervisor\n    consumers = [\n      [connection: MyConnection, queue: \"my_queue\", prefetch_count: 5],\n      [connection: MyConnection, queue: \"my_queue_2\", prefetch_count: 10],\n    ]\n\n    {:ok, consumers}\n  end\n\n  def init(:consumer, opts) do\n    # Perform runtime config per consumer\n    {:ok, opts}\n  end\n\n  @impl Rabbit.ConsumerSupervisor\n  def handle_setup(state) do\n    # Optional callback to perform any exchange or queue setup per consumer\n    AMQP.Queue.declare(state.channel, state.queue)\n    :ok\n  end\n\n  @impl Rabbit.ConsumerSupervisor\n  def handle_message(message) do\n    # Handle message consumption per consumer\n    IO.inspect(message.payload)\n    {:ack, message}\n  end\n\n  @impl Rabbit.ConsumerSupervisor\n  def handle_error(message) do\n    # Handle message errors per consumer\n    {:nack, message}\n  end\nend\n\nMyConsumerSupervisor.start_link()\n```\n\n## [Producers](https://hexdocs.pm/rabbit/Rabbit.Producer.html)\n\nIn order to publish messages to RabbitMQ, we must create a producer module. They\nmust be provided a connection module.\n\nUpon start, a producer will automatically create a pool of RabbitMQ channels\nto publish from.\n\nYou can optionally implement the `handle_setup/1` callback to perform any work\nneeded to declare queues/exchanges/bindings.\n\n```elixir\ndefmodule MyProducer do\n  use Rabbit.Producer\n\n  def start_link(opts \\\\ []) do\n    Rabbit.Producer.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n\n  # Callbacks\n\n  @impl Rabbit.Producer\n  def init(:producer_pool, opts) do\n    # Perform runtime config for the producer pool\n    {:ok, opts}\n  end\n\n  def init(:producer, opts) do\n    # Perform runtime config per producer\n    {:ok, opts}\n  end\nend\n\nMyProducer.start_link(connection: MyConnection)\nRabbit.Producer.publish(MyProducer, \"\", \"my_queue\", \"hello\")\n```\n\n## [Topology](https://hexdocs.pm/rabbit/Rabbit.Topology.html)\n\nTopology provides a way to centralize any RabbitMQ setup required by your\napplication. In that sense, it should be started BEFORE any of your producers\nor consumers.\n\nUsing a topology, you can automatically setup queues, exchanges and bindings\nwith simple keyword lists.\n\n```elixir\ndefmodule MyTopology do\n  use Rabbit.Topology\n\n  def start_link(opts \\\\ []) do\n    Rabbit.Topology.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n\n  # Callbacks\n\n  @impl Rabbit.Topology\n  def init(:topology, opts) do\n    # Perform runtime config\n    {:ok, opts}\n  end\nend\n\nMyTopology.start_link(\n  connection: MyConnection,\n  queues: [\n    [name: \"my_queue\", durable: true],\n    [name: \"my_queue_2\", durable: true],\n  ],\n  exchanges: [\n    [name: \"my_exchange\"],\n    [name: \"my_exchange_2\", type: :fanout, durable: true],\n  ],\n  bindings: [\n    [type: :queue, source: \"my_exchange\", destination: \"my_queue\", routing_key: \"my_key\"],\n    [type: :exchange, source: \"my_exchange_2\", destination: \"my_exchange_1\"]\n  ]\n)\n```\n\n## [Brokers](https://hexdocs.pm/rabbit/Rabbit.Broker.html)\n\nBrokers encapsulate all of the above components into a single easy-to-use module.\nIt provides a single place to handle your RabbitMQ connections, topology,\nproducers and consumers.\n\n```elixir\ndefmodule MyBroker do\n  use Rabbit.Broker\n\n  def start_link(opts \\\\ []) do\n    Rabbit.Broker.start_link(__MODULE__, opts, name: __MODULE__)\n  end\n\n  # Callbacks\n\n  @impl Rabbit.Broker\n  # Perform runtime configuration per component\n  def init(:connection_pool, opts), do: {:ok, opts}\n  def init(:connection, opts), do: {:ok, opts}\n  def init(:topology, opts), do: {:ok, opts}\n  def init(:producer_pool, opts), do: {:ok, opts}\n  def init(:producer, opts), do: {:ok, opts}\n  def init(:consumer_supervisor, opts), do: {:ok, opts}\n  def init(:consumer, opts), do: {:ok, opts}\n\n  @impl Rabbit.Broker\n  def handle_message(message) do\n    # Handle message consumption per consumer\n    IO.inspect(message.payload)\n    {:ack, message}\n  end\n\n  @impl Rabbit.Broker\n  def handle_error(message) do\n    # Handle message errors per consumer\n    {:nack, message}\n  end\nend\n\nMyBroker.start_link(\n  connection: [uri: \"amqp://guest:guest@127.0.0.1:5672\"],\n  topology: [\n    queues: [\n      [name: \"my_queue\", durable: true],\n      [name: \"my_queue_2\", durable: true]\n    ]\n  ],\n  producer: [pool_size: 10],\n  consumers: [\n    [queue: \"my_queue\"],\n    [queue: \"my_queue_2\", prefetch_count: 10]\n  ]\n)\nRabbit.Broker.publish(MyBroker, \"\", \"my_queue\", \"hello\")\n```","ref":"readme.html#documentation"}],"proglang":"elixir","producer":{"name":"ex_doc","version":"0.35.1"},"content_type":"text/markdown"}